/********************************************************************************
** CIS 15 BG-03
** Winter, 2004
***************
**
** Lab 5B       Strings - Hang Man         10 Points
**
****************************************************

  There are three requirements:
  
     1. Read the following program, 
        try to understand how it works and what it does (run it!), 
        and then draw the STRUCTURE CHART

     2. Write a SHORT REPORT, less than one page, about what you have
        learned from this program. Also, explain how would you make it
        better, what do you think needs to be added, etc.

     3. One of the improvements is to let the user play the game again:
        read a list of words from a text file WORDS.TXT, into an array, 
        and randomly choose one of them.

   Turn in the following items:
            . Front Page
            . Output (not more than two pages)
            . STRUCTURE CHART - 2 Points
            . Source File     - 1 Point - add your remarks next to the 
                                code fragment being analyzed
                                (in red, blue, or green).
                                5 Points - improvements 
            . SHORT REPORT    - 2 Points. 


****************************************
**
**  Written By: ...
**
**  Date: ...
**
********************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define WORD_SIZE 31
#define FLUSH while( getchar() != '\n' )
#define PRESS_ENTER printf( "Press ENTER to continue "), getchar()

void printGameInfo( void );
int  getLevel ( void );
void printMenu( void );
void printLevelInfo( int level, int scale[] );

int  playGame(  int level, int scale[], char *my_word );
void initializeWord( char *word, int length );
void printWord( char *word );
char getLetter( void );
void processWrongLetter( int *no_of_misses, int max_misses );
void processRightLetter( char *uniqueOpt, int  *guessed,      char *pLetter,
                         char *my_word,   char *guessed_word, char letter );
void printResult( int success, char *word );
void printEndGame( void );

int main ( void )
{
   int  scale[5] = {3, 6, 10, 16, 25};
   int  level;
   char my_word[WORD_SIZE] = "difficulty";
   int  success;

   printGameInfo();   
   level = getLevel();
   printLevelInfo( level, scale );
   success = playGame( level, scale, my_word );
   printResult( success, my_word );
   printEndGame();

   return 0;

} /* End main */

/* ===================== getLevel ===================== */
/* 
      Pre :
      Post:
*/
void printGameInfo( void )
{
   system( "cls" );
   printf( "\n\n\n\tThis is a popular children's game:"
           "\n\n\n\t\t H A N G M A N!\n\n"
           "\tThe computer will generate a word and\n"
           "\tthe player will try to guess it!\n\n\n\n"
           "\tThe game terminates\n\n"
           "\t\t- when the correct word is guessed or\n\n"
           "\t\t- when a predetermined number of incorrect\n"
           "\t\t  choices of letters have been made.\n\n\n\n\n" );
   PRESS_ENTER;

   return;

} /* End greeting */

/* ===================== getLevel ===================== */
/* 
      Pre :
      Post:
*/
int getLevel( void )
{
   int level;

   printMenu();
   printf( "\t Choose the level of difficulty: " );
   scanf( "%d", &level );
   FLUSH;
   while( level < 1 || level > 5 )
   {
      printf( "\n\n\n\t *** Invalid option! Try again! ***\n\n\t     " );
      PRESS_ENTER;
      printMenu();
      printf( "\t Choose the level of difficulty: " );
      scanf( "%d", &level );
      FLUSH;
   }
    
   return level;

} /* End getLevel */

/* ===================== printMenu ===================== */
/* 
      Pre :
      Post:
*/
void printMenu( void )
{
   system ( "cls" );
   printf( "\n\n\n\t\t\t H A N G M A N \n\n\n" );
   printf( "\t Level of difficulty           Number of Misses\n\n" );
   printf( "\t     1. Expert                       0 -  3 \n"
           "\t     2. Very good                    4 -  6 \n"
           "\t     3. Average                      7 - 10 \n"
           "\t     4. Beginner                    11 - 16 \n"
           "\t     5. Needs Practice              17 - 25 \n\n" );

   return;

} /* End printMenu */


/* ===================== printLevelInfo ===================== */
/* 
      Pre :
      Post:
*/
void printLevelInfo( int level, int scale[] )
{

   system( "cls" );
   printf( "\n\n\n\t\tLevel %d:\n\n\t\t\t", level );
   switch( level )
   {
      case 1: printf( "You are an expert!\n"); 
              break;
      case 2: printf( "You are a very good player!\n");
              break;
      case 3: printf( "You are an average player!\n");
              break;
      case 4: printf( "You are a beginner!\n");
              break;
      case 5: printf( "You need more practice!\n");
   }
   printf( "\n\n\n\t\tThe number of letters you may miss is %d!\n\n\n", scale[level - 1] ); 
   printf( "\t\tGood luck!\n\n\n\n\n" );
   PRESS_ENTER;
} /* End printLevelInfo */

/* ===================== playGame ===================== */
/* 
      Pre :
      Post:
*/
int  playGame(  int level, int scale[], char *my_word )
{
   int  number_of_misses;
   int  max_misses;
   int  word_length;
   int  guessed;
   char letter;
   char unique_opt[WORD_SIZE] = "";
   char guessed_word[WORD_SIZE];
   char *pLetter;

   system( "cls" );
   word_length = strlen( my_word );
   initializeWord( guessed_word, word_length );
   printf( "\n\n\n The unknown word has %d letters:\n\n", word_length );

   max_misses       = scale[level - 1];
   number_of_misses = 0;
   guessed          = 0;
   while( number_of_misses < max_misses && guessed < word_length )
   {
      letter = getLetter();
      pLetter = strchr( my_word, letter );
      if( pLetter == NULL )
         processWrongLetter( &number_of_misses, max_misses );
      else
      {
         if( strchr( unique_opt, letter ) ) /* do not enter the same letter twice! */
            processWrongLetter( &number_of_misses, max_misses );
         else
            processRightLetter( unique_opt, &guessed, pLetter, my_word, guessed_word, letter );
      }   
      printWord( guessed_word );
   }
        
   return guessed == word_length;

} /* End playGame */

/* ===================== initializeWord ===================== */
/* 
      Pre :
      Post:
*/
void initializeWord( char *word, int length )
{
   char *pLast;
   
   for( pLast = word + length; word < pLast; word++ )
      *word = ' ';
   *word = '\0';

   return;
}

/* ===================== printWord ===================== */
/* 
      Pre :
      Post:
*/
void printWord( char *word )
{
    
   printf( "\t\t\t " );

   for(  ; *word != '\0'; word++ )
      if( *word == ' ' )
         printf( "_ " );
      else
         printf( "%c ", toupper( *word ) );

   printf( "\n" );

   return;
}

/* ===================== getLetter ===================== */
/* 
      Pre :
      Post:
*/

char getLetter( void )
{
    char letter;

    printf( "Enter a letter: " );
    scanf( " %c", &letter );
    letter = tolower( letter );

    return letter;
}

/* ===================== processWrongLetter ===================== */
/* 
      Pre :
      Post:
*/
void processWrongLetter( int *number_of_misses, int max_misses )
{
   int mistakes;

   printf( "\a - Wrong!\n" );
   (*number_of_misses)++;     
   mistakes = max_misses - *number_of_misses;
   if( mistakes )                                /* printing letters or letter */
      printf( "You may miss %d letter%s\n",        
                   mistakes, 
                   mistakes == 1 ? "." : "s." ); 
   return;
} /* End processWrongLetter */

/* ===================== processRightLetter ===================== */
/* 
      Pre :
      Post:
*/
void processRightLetter( char *unique_opt, int  *guessed,      char *pLetter,
                         char *my_word,    char *guessed_word, char letter )

{
   sprintf( unique_opt, "%s%c", unique_opt, letter );
   printf( " + Right!\n\n" );
   (*guessed)++;
   *(guessed_word + (pLetter - my_word) ) = letter;
   while( (pLetter = strchr( pLetter + 1, letter ))!= NULL )
   {
      (*guessed)++;
      *(guessed_word + (pLetter - my_word) ) = letter;
   }
   return;
} /* End processRightLetter */

/* ===================== printResult ===================== */
/* 
      Pre :
      Post:
*/
void printResult( int success, char *word )
{
   if( success )
      printf( "\n\t\t\t Congratulations! You did it!\n" );
   else
   {
      printf( "Sorry, the word was:\n" );
      printWord( word );
   }

   return;
}

/* ===================== printEndGame ===================== */
/* 
      Pre :
      Post:
*/
void printEndGame( void )
{
   
   printf( "\n\n\n\n\t\t\t *** T H E    END ***\n\n\n" );

   return;
}
